<?php

namespace BackOfficeOptimizer\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class ProcedureCaller extends \Teknisa\Libs\Controller\CRUD {

    protected $newCode;
    protected $lista;

    function __construct($crudService, $environment, $newCode, $connection, $lista)
    {
        parent::__construct($crudService, $environment);
        $this->newCode = $newCode;
        $this->connection = $connection;
        $this->lista = $lista;
    }

    // GET ERROR PROCEDURES IN LISTAERROS
    function getErrosProcedure($codeError) {
        $sql = "SELECT DSERRO FROM LISTAERROS WHERE NRLISTA = :P_LISTA_ERROS ORDER BY CDCHAVE";
        $queryParameters = array( "P_LISTA_ERROS" => $codeError );

        $query = $this->connection->prepare($sql);
        $query->execute($queryParameters);
        return $query->fetchAll();
    }

    /// EXAMPLES OF PROCEDURE CALLER
    function exportValidityCaller(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrlista = $this->lista->generateNewList($row['NRORG'], $row['OBJECTEXPORT']['lista']);
        
        if($row['OBJECTEXPORT']['typeProcedure'] == 'UNITY') {
            $resProcedure = $this->exportValidityProcedure($row, $nrlista);
        } else if($row['OBJECTEXPORT']['typeProcedure'] == 'VALIDITY') {
            $listaVigencia = $this->lista->generateNewList($row['NRORG'], array($row['CDFILIAL'].$row['CDSERVFILI'].$row['OBJECTEXPORT']['validityDest']));
            $resProcedure = $this->exportItemValidityProcedure($row, $nrlista, $listaVigencia);
            $this->lista->deleteLista($listaVigencia);
        }
        $this->lista->deleteLista($nrlista);
        $response->addDataset(new DataSet('status', $resProcedure));
    }
    
    private function getItemsFromBatch( $NRGERACAOCARD ){
        $SQL = <<<SQL
            SELECT
                NRITGERACAOCARD
            FROM
                ITGERACAOCARD
            WHERE
                NRGERACAOCARD = :P_NRGERACAOCARD
SQL;
        $stmt = $this->connection->prepare($SQL);
        $stmt->execute([
            'P_NRGERACAOCARD' => $NRGERACAOCARD
        ]);
        return array_column($stmt->fetchAll(), 'NRITGERACAOCARD');
    }

    function newBatchProcedureCaller(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrlista = $this->lista->generateNewList($row['NRORG'], $row['OBJECTNEWBATCH']['SERVICES']);
        $NRGERACAOCARD = $this->newBatchProcedure($row, $nrlista);
        $this->lista->deleteLista($nrlista);
        $response->addDataset(new DataSet('status', ['NRGERACAOCARD' => $NRGERACAOCARD]));
    }

    function newBatchProcedure($row, $nrlista)
    {
        $procedure = new StoredProcedure($this->connection, 'INSERE_LOTE_OTIMIZADOR');
        $procedure->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTMESCARD', Param::PARAM_INPUT, $row['OBJECTNEWBATCH']['MONTH'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTIPORESTRICAO', Param::PARAM_INPUT, $row['TPCONTRATO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREFERENCIA', Param::PARAM_INPUT, $row['OBJECTNEWBATCH']['DTREFERENCIA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOCUSTO', Param::PARAM_INPUT, $row['OBJECTNEWBATCH']['CDTIPOCUSTO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDAGRUPPROD', Param::PARAM_INPUT, $row['OBJECTNEWBATCH']['CDAGRUPPROD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTE', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_VERIFICA_ALERTA', Param::PARAM_INPUT, $row['OBJECTNEWBATCH']['VERIFY_ALERT'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VERIFICA_CUSTO', Param::PARAM_INPUT, $row['OBJECTNEWBATCH']['VERIFY_COST'], Param::PARAM_TYPE_STR));
        
        $result = $procedure->execute();
        if($result && isset($result['P_NRLOTE'])){
            return $result['P_NRLOTE'];
        }
        return array();
    }

    function exportCadOtimizadorCaller(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $nrlistaItem = $this->lista->generateNewList($row['NRORG'], $row['ITEM']);
        $nrlistaServ = $this->lista->generateNewList($row['NRORG'], $row['FILISERV']);
        
        $procedure = $this->exportCadOtimizadorProcedure($row, $nrlistaItem, $nrlistaServ);
        
        $this->lista->deleteLista($nrlistaItem);
        $this->lista->deleteLista($nrlistaServ);
        $response->addDataset(new DataSet('data', $procedure));
    }

    // EXAMPLE OF PROCEDURE
    function exportValidityProcedure($objectExport, $nrlista)
    {
        $procedure = new StoredProcedure($this->connection, 'EXPORTA_VIGENCIAS_OTIMIZADOR');
        $procedure->addParam(new Param('P_TABELA', Param::PARAM_INPUT, $objectExport['OBJECTEXPORT']['table'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRVIGENCIA', Param::PARAM_OUTPUT, $objectExport['OBJECTEXPORT']['vigencia'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $objectExport['CDFILIAL'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $objectExport['CDSERVFILI'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDTIPORESTRICAO', Param::PARAM_INPUT, $objectExport['IDTIPORESTRICAO'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDOPCAO', Param::PARAM_INPUT, $objectExport['OBJECTEXPORT']['opcao'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $objectExport['CDOPERADOR'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_LISTA_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
        
        $v_error = $procedure->execute();
        if(!empty($v_error)) {
            return $this->getErrosProcedure($v_error['P_LISTA_ERROS']);
        }

        return array();
    }

    function exportItemValidityProcedure($objectExport, $nrlista, $listaVigencia)
    {
        $procedure = new StoredProcedure($this->connection, 'EXPORTA_VIG_ITEM_OTIMIZADOR');
        $procedure->addParam(new Param('P_TABELA', Param::PARAM_INPUT, $objectExport['OBJECTEXPORT']['table'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRVIGENCIA', Param::PARAM_OUTPUT, $objectExport['OBJECTEXPORT']['vigencia'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $objectExport['CDFILIAL'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $objectExport['CDSERVFILI'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDTIPORESTRICAO', Param::PARAM_INPUT, $objectExport['IDTIPORESTRICAO'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRLISTA_DEST', Param::PARAM_INPUT, $listaVigencia, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $nrlista, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDOPCAO', Param::PARAM_INPUT, $objectExport['OBJECTEXPORT']['opcao'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $objectExport['CDOPERADOR'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_LISTA_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
        $procedure->execute();
        return array();
    }

    function exportCadOtimizadorProcedure($objectExport, $nrlistaItem, $nrlistaServ)
    {
        $procedure = new StoredProcedure($this->connection, 'EXPORTA_CADASTROS_OTIMIZADOR');
        $procedure->addParam(new Param('P_TABELA', Param::PARAM_INPUT, $objectExport['TABELA'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRLISTAITEM', Param::PARAM_OUTPUT, $nrlistaItem, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $objectExport['CDFILIAL'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $objectExport['CDSERVFILI'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDTIPORESTRICAO', Param::PARAM_INPUT, $objectExport['IDTIPORESTRICAO'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_NRLISTAFILISERV', Param::PARAM_INPUT, $nrlistaServ, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDOPCAO', Param::PARAM_INPUT, $objectExport['MODO'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $objectExport['CDOPERADOR'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_LISTA_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_CDGRUPOPRATODEST', Param::PARAM_INPUT, $objectExport['PLATEGRID']['GROUPARR'], Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDEXCLUIORGIEM', Param::PARAM_INPUT, $objectExport['PLATEGRID']['MODE'], Param::PARAM_TYPE_STR, 10));
        
        $v_error = $procedure->execute();
        
        if(!empty($v_error)) {
            return $this->getErrosProcedure($v_error['P_LISTA_ERROS']);
        }

        return array();
    }
}